/*  
 *  Copyright (C) International Business Machines  Corp., 2005 
 *  Author:     Michael Running Wolf <marunnin@us.ibm.com> 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  XML-RPC xenstat method functions, 
 * 
 */

#include "xenstatd.h"

xmlrpc_value * build_vcpu_struct(xmlrpc_env * env, xenstat_vcpu * vcpu,int id)
{
	xmlrpc_value * xmlrpc_result;
	char vcpu_ns_str[STR_SIZE];	   

	sprintf(vcpu_ns_str,"%llu",xenstat_vcpu_ns(vcpu));
     
	xmlrpc_result = xmlrpc_build_value(env, "{s:i,s:s}",
			   "id",id,
			   "ns",vcpu_ns_str
			   );

	return xmlrpc_result;
}

xmlrpc_value * build_network_struct(xmlrpc_env * env, xenstat_network * network)
{

    xmlrpc_value * xmlrpc_result;
    int network_id;
    char rbytes_str[STR_SIZE], rpackets_str[STR_SIZE], rerrs_str[STR_SIZE], rdrop_str[STR_SIZE],
        tbytes_str[STR_SIZE], tpackets_str[STR_SIZE], terrs_str[STR_SIZE], tdrop_str[STR_SIZE];
    
	   network_id = xenstat_network_id(network);

	   sprintf(rbytes_str,"%llu",xenstat_network_rbytes(network));
	   sprintf(rpackets_str,"%llu",xenstat_network_rpackets(network));
	   sprintf(rerrs_str,"%lld",xenstat_network_rerrs(network));
	   sprintf(rdrop_str,"%llu",xenstat_network_rdrop(network));

	   sprintf(tbytes_str,"%llu",xenstat_network_tbytes(network));
	   sprintf(tpackets_str,"%llu",xenstat_network_tpackets(network));
	   sprintf(terrs_str,"%llu",xenstat_network_terrs(network));
	   sprintf(tdrop_str,"%llu",xenstat_network_tdrop(network));

	   xmlrpc_result = xmlrpc_build_value(env, "{s:i,s:s,s:s,s:s,s:s,s:s,s:s,s:s,s:s}",
			   "id",network_id,
			   "rbytes",rbytes_str,
			   "rpackets",rpackets_str,
			   "rerrs",rerrs_str,
			   "rdrop",rdrop_str,
			   "tbytes",tbytes_str,
			   "tpackets",tpackets_str,
			   "terrs",terrs_str,
			   "tdrop",tdrop_str
			   );

	return xmlrpc_result;
}

xmlrpc_value * build_domain_struct(xmlrpc_env * env, xenstat_domain * domain)
{
	xmlrpc_value * xmlrpc_result, * vcpu_array,* network_array;
	int id, ssid, running, paused, blocked, crashed, shutdown, dying, 
	    num_vcpus, num_networks;
	char max_mem_str[STR_SIZE], cpu_ns_str[STR_SIZE], cur_mem_str[STR_SIZE];
	
     id = xenstat_domain_id(domain);
     ssid = xenstat_domain_ssid(domain);

     sprintf(max_mem_str,"%llu",xenstat_domain_max_mem(domain));
     sprintf(cpu_ns_str,"%llu", xenstat_domain_cpu_ns(domain));
     sprintf(cur_mem_str,"%llu", xenstat_domain_cur_mem(domain));

     num_vcpus = xenstat_domain_num_vcpus(domain);
	vcpu_array = build_vcpu_array(env,domain);
     num_networks = xenstat_domain_num_networks(domain);
	network_array = build_network_array(env,domain);

     running = xenstat_domain_running(domain);
     paused = xenstat_domain_paused(domain);
     blocked = xenstat_domain_blocked(domain);
     crashed = xenstat_domain_crashed(domain);
     shutdown = xenstat_domain_shutdown(domain);
     dying = xenstat_domain_dying(domain);

     xmlrpc_result = xmlrpc_build_value(env,
			   "{s:i,s:s,s:{s:i,s:s,s:s,s:i,s:s,s:i,s:V,s:V},s:{s:b,s:b,s:b,s:b,s:b,s:b}}",
			   "id",id,
			   "name","monkey",
			   "resources",
			        "ssid",ssid,
				"max_mem",max_mem_str,
				"cur_mem",cur_mem_str,
				"num_vcpus",num_vcpus,
				"cpu_ns",cpu_ns_str,
			        "num_networks",num_networks,
				"vcpus",vcpu_array,
				"networks",network_array,
                             "states",
			     "running",running,
			     "paused",paused,
			     "blocked",blocked,
			     "crashed",crashed,
			     "shutdown",shutdown,
			     "dying",dying
			   );
	xmlrpc_DECREF(network_array);
	xmlrpc_DECREF(vcpu_array);

	return xmlrpc_result;
}

xmlrpc_value * build_machine_stat_struct(xmlrpc_env * env, xenstat_node * node)
{
    xmlrpc_value * xmlrpc_result, *domain_array;
    int num_cpus, num_domains, time_stamp;
    char cpu_hz_str[STR_SIZE], tot_mem_str[STR_SIZE], free_mem_str[STR_SIZE];

    domain_array = build_domain_array(env,node);

    num_domains = xenstat_node_num_domains(node);
    num_cpus = xenstat_node_num_cpus(node);
    time_stamp = 1000;
    sprintf(cpu_hz_str,"%llu", xenstat_node_cpu_hz(node));
    sprintf(tot_mem_str,"%llu", xenstat_node_tot_mem(node));
    sprintf(free_mem_str,"%llu", xenstat_node_free_mem(node));

    xmlrpc_result = xmlrpc_build_value(env,"{s:i,s:s,s:{s:i,s:s,s:s,s:s,s:i,s:s},s:V}",
	                 "timestamp",	time_stamp,
				  "hostname",	"monkey",
				  "machine_stats",
				    "num_cpus",	num_cpus,
				    "cpu_hz",cpu_hz_str,
				    "tot_mem",tot_mem_str,
				    "free_mem",free_mem_str,
				    "num_domains",num_domains,
				    "xen_version","xen-moo-1.2.0",
				   "domains",domain_array
					);

    xmlrpc_DECREF(domain_array);
	return xmlrpc_result;
}

xmlrpc_value * build_vcpu_array(xmlrpc_env * env, xenstat_domain * domain)
{
	int i;
	xmlrpc_value * xmlrpc_result, * cur_vcpu;
	xenstat_vcpu * xen_vcpu;

     xmlrpc_result = xmlrpc_build_value(env,"()");

	for(i = 0; i < xenstat_domain_num_vcpus(domain); i++){
	   xen_vcpu = xenstat_domain_vcpu(domain,i); 
	   cur_vcpu = build_vcpu_struct(env,xen_vcpu,i); 
	   xmlrpc_array_append_item(env,xmlrpc_result,cur_vcpu);	    
        xmlrpc_DECREF(cur_vcpu);
	}

	return xmlrpc_result;
}
xmlrpc_value * build_network_array(xmlrpc_env * env, xenstat_domain * domain)
{
	int i;
	xmlrpc_value * xmlrpc_result, * cur_network;
	xenstat_network * xen_network;

     xmlrpc_result = xmlrpc_build_value(env,"()");
	for(i = 0; i < xenstat_domain_num_networks(domain); i++){
	   xen_network = xenstat_domain_network(domain,i); 
	   cur_network = build_network_struct(env,xen_network);
        xmlrpc_array_append_item(env,xmlrpc_result,cur_network);	    
        xmlrpc_DECREF(cur_network);
	}

	return xmlrpc_result;

}
xmlrpc_value * build_domain_array(xmlrpc_env * env, xenstat_node * node)
{
	int i;
	xmlrpc_value *xmlrpc_result, *cur_domain, *vcpu_array, *network_array;
	xenstat_domain * domain;

     xmlrpc_result = xmlrpc_build_value(env,"()");

    for(i = 0; i < xenstat_node_num_domains(node); i ++){
        domain = xenstat_node_domain_by_index(node, i);

	   if(domain== NULL){
            fprintf(stderr, "\nFailed to get domain information\n");
            return NULL;
	   }

	     cur_domain = build_domain_struct(env, domain);	

          xmlrpc_array_append_item(env,xmlrpc_result,cur_domain);	 
		xmlrpc_DECREF(cur_domain);
    }

	return xmlrpc_result;
}

/**/
/*void *         const user_data ATTR_UNUSED) {*/

/*
<<<<<<< xenstatd.c
=======
static xmlrpc_value * xenstat_gather(xmlrpc_env *   const env, 
                                     xmlrpc_value * const param_array, 
                                     void *         const user_data __attribute__((__unused__))) {
*/

xmlrpc_value * xenstat_gather(xmlrpc_env *   const env,
                                     xmlrpc_value * const param_array,
                                     void *         const user_data __attribute__((__unused__))){

    xmlrpc_value *xenstat_xmlresults; //pointer to returned xml package

    xenstat_node *node;

    
    printf("prepping data\n");
    //handle = xenstat_init();
     
     
    printf("UID: %d\n", getuid());
    if (handle == NULL) {
	fprintf(stderr, "Failed to init xenstat \n");
      //FIX ME return xmlrpc error package here 
	return NULL; 
    }

    printf("getting node information\n");
    node = xenstat_get_node(handle, XENSTAT_ALL); 
    if (node == NULL) {
	fprintf(stderr, "Failed to get node information\n");
      //FIX ME return xmlrpc error package here 
	return NULL; 
    }

    printf("\nbuilding xmlrpc reply\n");
    xenstat_xmlresults = build_machine_stat_struct(env, node);

    printf("sending result\n"); 
    xenstat_free_node(node);

    return  xenstat_xmlresults;
     
/*    return xmlrpc_build_value(env, "i", 2);
 */

}
/**/

/*
>>>>>>> 1.7
static xmlrpc_value *
sample_add(xmlrpc_env *   const env, 
		           xmlrpc_value * const param_array, 
                                     void *         const user_data) {

   xmlrpc_int32 x, y, z;

       xmlrpc_decompose_value(env, param_array, "(ii)", &x, &y);
    if (env->fault_occurred)
           return NULL;

       z = x + y;

    return xmlrpc_build_value(env, "i", z);
}
*/

